/**
 * Global variables
 */
var dsid; // dariah storage id
var map; // osm
var proj4326; // osm projection
var projmerc; // mercator projection
var locationPickerPoint;
var locationPickerLayer;
var dragFeature;
var autosave = true; // We do need to turn off autosave from time to time.

/**
 * Stuff to setup when DOM is ready
 */
$(document).ready(function(){
  initTable(); // Init the table widget.
  initMap(); // Init the OpenStreetmap widget.
  initClipboard(); // Init clipboard and tooltips.
  init();
});

/**
 *
 */
function logWidth() {
  console.log("offwi: " + $('#sheetArea')[0].offsetWidth);
  console.log("wi: " + $('#sheetArea').width());
  //console.log("wi css: " + $('#console').css(width));
  console.log("owwi: " + $('#sheetArea').outerWidth(true));

  console.log("dt offwi: " + $('#dataTable')[0].offsetWidth);
  console.log("dt wi: " + $('#dataTable').width());
  //console.log("wi css: " + $('#console').css(width));
  console.log("dt owi: " + $('#dataTable').outerWidth(true));
}

/**
 * Main init method, called every time the page is reloaded!
 */
function init() {

    // Check and fill logged in field and disabe buttons if not logged in.
    /*
    if (sessionStorage.getItem('tok')) {
        $('#loggedin').text('You are loggged in');
    } else {
        $('#loggedin').text('You are not loggged in');
    }
    */

    // Do things depending on storage ID existance.
    doThingsOnIDExistingCheck();

    // Set token first if token hash (#access_token=...) is existing.
    if (window.location.hash) {
        // Get complete token hash from PDP.
        var acctok = window.location.hash.substr(1);
        // Get pure token out of complete token hash.
        var puretok = acctok.substr(acctok.indexOf("access_token=") + 13, acctok.indexOf("&") - 13);
        sessionStorage.setItem('tok', puretok);
        // Remove token hash from URL.
        window.location = window.location.origin + window.location.pathname + window.location.search;
    }
    else {
	    // Drag and drop handler for CSV documents to workarea.
	    $('#workArea')
		    .bind('dragover', handleDragOver, false)
		    .bind('drop', handleDropFileSelect);

	    // Action for fileuploadbutton.
	    $('#files').bind('change', handleFileSelect);
        $('.downloadLink').click(download);

		// jQuery creates it's own event object, and it doesn't have a dataTransfer property yet.
        // This adds dataTransfer to the event object. Thanks to l4rk for figuring this out!
		jQuery.event.props.push('dataTransfer');

        // New datasheet shall be created.
        if ($.query.get('action') === 'create') {
            createNewDataset();
        }
        else {
            // If we have got an ID in the URL params (no create action).
            if ($.query.get('id') !== '') {
                dsid = $.query.get('id');
                // Check share/unshare status here.
                checkSharedStatus(dsid);
                // Load data from storage if id supplied in params.
		        loadFromDariahStorage(dsid);
		        document.title = document.title + ' ' + dsid;
            	url = storageURL + dsid;
            	$('#dataLoc input').val(dsid);
            	$('#dataLoc button').attr('data-clipboard-text', url);
	    	}
            // If we have got NO id in the URL params (no create action).
        	else {
	            showFirstStartWizard();
    	    }
    	}
	}
}

/**
 *
 */
function createNewDataset() {
    if (sessionStorage.getItem('tok') !== null) {
        // Create the new handsontable here.
        $('#dataTable').handsontable('loadData', [e4dHeader]);
        // Put table to csv array and create CSV structure.
        var arr = $('#dataTable').handsontable('getDataReference');
    	var csv = arr2csv(arr);
        // Create new data file.
        postToDariahStorage(csv);
    } else {
        authenticate('create');
    }
}

/*
 *
 */
function deleteDataset() {
    if (sessionStorage.getItem('tok') !== null) {
        $('#askForDeletion').modal('show');
    } else {
        authenticate('delete');
    }
}

/*
 *
 */
function shareDataset() {
    if (sessionStorage.getItem('tok') !== null) {
        $('#askForSharing').modal('show');
    } else {
        authenticate('share');
    }
}

/*
 *
 */
function unshareDataset() {
    if (sessionStorage.getItem('tok') !== null) {
        $('#askForUnsharing').modal('show');
    } else {
        authenticate('unshare');
    }
}

/*
 * If we can read the dataset without token, it is public :-)
 */
function checkPublicAccess() {
    if (sessionStorage.getItem('tok') !== null) {
        var response = checkStorageAccess('delete');
    } else {
        authenticate('unshare');
    }
}

/**
 *
 */
var saveTimer;
function autoSave(change) {
    if (autosave) {
        var arr = $('#dataTable').handsontable('getDataReference');
    	var csv = arr2csv(arr);
        // Make sure that not more than one save per 2,5 seconds occur.
        clearTimeout(saveTimer);
        saveTimer = setTimeout(function() {
            updateDariahStorage(dsid, csv);
        }, 2500);
    }
}

/**
 * Functions for loading file into Browser, drag'n'drop or with button
 * look at
 * - http://www.html5rocks.com/de/tutorials/file/dndfiles/
 * - http://weblog.bocoup.com/using-datatransfer-with-jquery-events/
 */

 /**
  *
  */
function handleFileSelect(evt) {
	var files = evt.target.files; // FileList object
	loadFromFiles(files);
}

/**
 *
 */
function handleDropFileSelect(evt) {
	evt.stopPropagation();
	evt.preventDefault();
	var files = evt.dataTransfer.files; // FileList object.
	loadFromFiles(files);
}

/**
 *
 */
function handleDragOver(evt) {
	evt.stopPropagation();
	evt.preventDefault();
	evt.dataTransfer.dropEffect = 'copy'; // Explicitly show this is a copy.
}

/**
 *
 */
function loadFromFiles(files) {
	for (var i = 0, f; f = files[i]; i++) {
		console.log(f.name + ' - ' + f.type || 'n/a' + ' - ' + f.size + ' bytes, last modified: ' + f.lastModifiedDate.toLocaleDateString());

		if($.inArray(f.type, allowedMimeTypes) < 0 ) {
            newAlert('error', 'Could not open file', 'Mime-Type "' + f.type + '" not supported. If your file is a CSV file, maybe it is missing a .csv file suffix?', '');
			continue;
		}

		var reader = new FileReader();
		reader.onload = function(e) {
			var data = e.target.result;
			csvToTable(data);
			postToDariahStorage(data);
		};

		reader.readAsText(f);
	}
}

/**
 *
 */
function csvToTable(data) {
	var arr = jQuery.csv()(data);
	// First turn off autosave.
	autosave = false;
	$('#dataTable').handsontable('loadData', arr);
	// colHeaders kind of broken...? this fixes, but what's the reason?
	$('#dataTable').handsontable("updateSettings", {colHeaders: true, rowHeaders:true });
    // Turn on autosave again.
    autosave = true;
	// Show alert.
	hideFirstStartWizard();
}

/**
 *
 */
function download() {
  var csv = encodeURI(arr2csv($('#dataTable').handsontable('getDataReference')));
  $(this).attr('href', 'data:text/csv;charset=utf-8,' + csv);
  $(this).attr('download', dsid + '.csv');

  console.log("dl: ", csv);
}

/**
 *
 */
function hideFirstStartWizard() {
	$('#firstStartWizard').hide();
	$('#sheetArea').show();
    showMessage('console', 'All changes will be autosaved.');
    showMessage('idMessage', 'Your dataset is stored to the DARIAH-DE OwnStorage and has got the ID');
	// Check if data in sheet is fine.
	//checkColumnHeaders();
}

/**
 *
 */
function showFirstStartWizard() {
	$('#firstStartWizard').show();
	$('#sheetArea').hide();
    showMessage('console', '');
    showMessage('idMessage', '');
    showMessage('sharedStatus', '');
}

/**
 *
 */
function newSheetAlert() {
	hideFirstStartWizard();
    var url = host + '/edit/index.html?id=' + dsid;
	var title = 'New datasheet created!';
	var message = '<p>Your datasheet has been successfully created and stored to the DARIAH-DE OwnStorage. '
        + 'You can access it here in the Datasheet Editor using the URL: ' + '<a href="'+ url + '">' + url + '</a></p>';
	newAlert('success', title,  message);
}

/**
 *
 */
function openGeoBrowser(newWindow) {
    // check all at once
    var val1 = checkColumnHeaders();
    var val2 = checkDates();
    var val3 = checkCoordinates();
    if (! (val1 && val2 && val3) ) return;

    //if (!checkDates()) return;
    //if (!checkCoordinates()) return;
	var url = e4dURL+'?csv1='+(encodeURIComponent(storageURL+dsid));
	console.log(url);
	if(newWindow) {
		window.open(url);
	} else {
		document.location.href = url;
	}
}

/**
 *
 */
function newAlert (type, title, message, extraClass) {
	if (extraClass === undefined) extraClass = '';

	elem = '<div class="alert alert-' + type + ' alert-block fade in ' + extraClass + ' ">'
        + '<a class="close" data-dismiss="alert" href="#">×</a>';
	if (title !== "") elem += '<h3 class="">' + title + '</h3>';
	elem += message + '</div>';

    $("#alert-area").append($(elem));
}

/**
 *
 */
function showMessage(area, text) {
	$('#' + area).text(text);
}


/**
 *
 */
function doThingsOnIDExistingCheck () {
    if (dsid) {
        url = storageURL + dsid;
        $('#dataLoc input').val(dsid);
        $('#dataLoc button').data('clipboard-text', url);
        $('#dataLoc button').removeClass('disabled');
        enableManagedButtons();
    } else {
        $('#dataLoc input').val('ID not (yet) set');
        $('#dataLoc button').addClass('disabled');
        disableManagedButtons();
    }
}

/**
 *
 */
function disableManagedButtons() {
    $('#downloadButton').addClass('disabled');
    $('#deleteButton').addClass('disabled');
    $('#shareButton').addClass('disabled');
    $('#unshareButton').addClass('disabled');
}

/**
 *
 */
function enableManagedButtons() {
    $('#downloadButton').removeClass('disabled');
    $('#deleteButton').removeClass('disabled');
    $('#shareButton').removeClass('disabled');
    $('#unshareButton').removeClass('disabled');
}

/**
 *
 */
function setSharedStatus() {
    $('#shareButton').hide();
    $('#unshareButton').show();
    $('#sharedStatus').text('Your dataset is publicly shared');
}

/**
 *
 */
function setUnsharedStatus() {
    $('#unshareButton').hide();
    $('#shareButton').show();
    $('#sharedStatus').text('Your dataset is private');
}

/**
 *
 */
function initClipboard() {
    $('button').tooltip('hide');
    var clipboard = new ClipboardJS('.doi-copy');
    clipboard.on('success', function(e) {
        var originalTitle = e.trigger.getAttribute('data-original-title');
        $(e.trigger).tooltip('hide')
            .attr('data-original-title', 'Copied')
            .tooltip('show')
        e.clearSelection();
        setTimeout(function() {
            $(e.trigger).tooltip('hide').attr('data-original-title', originalTitle);
        }, 2000);
    });
}
